#![no_std]
#![no_main]

use defmt_rtt as _;
use embassy_nrf as _;
use embassy_time::Delay;
use embedded_hal_async::delay::DelayNs;
use panic_probe as _;

use crate::lcd::Lcd;
use defmt::{error, info, unwrap};
use embassy_executor::Spawner;

pub mod ble {
    pub mod advertisment;
    pub mod battery_service;
    pub mod gatt;
    pub mod softdevice;
}

pub mod lcd;
pub mod peripherals;

const DEVICE_NAME: &str = "alert-me display";

#[embassy_executor::task]
async fn ble(spawner: Spawner) {
    let (sd, server) = ble::softdevice::run(spawner, DEVICE_NAME);

    loop {
        let conn = ble::advertisment::run(sd, DEVICE_NAME).await;
        let error = ble::gatt::run(&conn, &server).await;
        info!("gatt_server run exited with error: {:?}", error);
    }
}

#[embassy_executor::task]
async fn tick(mut lcd: lcd::Lcd<'static>, delay: u32) {
    let mut d = Delay {};
    loop {
        lcd.scroll_buffer()
            .await
            .unwrap_or_else(|e| error!("Failed to scroll lcd: {}", e));

        d.delay_ms(delay).await;
    }
}

#[embassy_executor::main]
async fn main(spawner: Spawner) {
    let peripherals = peripherals::init();
    let mut lcd = Lcd::new(
        0x27,
        peripherals.TWISPI0,
        peripherals.P0_03,
        peripherals.P0_04,
    );
    lcd.init()
        .await
        .unwrap_or_else(|e| error!("Failed to initialize lcd: {}", e));
    lcd.write("Hello, horse :)", lcd::Lines::First)
        .await
        .unwrap();
    lcd.write("I am an electronic horse!", lcd::Lines::Second)
        .await
        .unwrap();

    unwrap!(spawner.spawn(ble(spawner)));
    unwrap!(spawner.spawn(tick(lcd, 300)));
}
